#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys
import argparse
from collections import defaultdict

usage = """ 

Takes as input an ANNOTATED_TABULATED_FILE annotated with MWEs, in TSV format, 
and outputs the set of unique annotated MWEs

If a CONLL_FILE is provided (-c option), the components of the VMWEs are the lemmas found in the conll file
otherwise they are the tokens found in the ANNOTATED_TABULATED_FILE
 """

# read arguments
parser = argparse.ArgumentParser(usage = usage)
parser.add_argument('annotated_tabulated_file', help = 'tabulated file annotated with mwes', type=argparse.FileType('r'), default = None)
parser.add_argument('-c','--conll_file', help = 'Conll file parallel to the annotated_tabulated_file, used to provide the lemmas of the components', type=argparse.FileType('r'), default = None)
parser.add_argument('-i','--column_for_annotated_mwe_ids', help = 'column number in TSV for the MWEs ids. Default=4', type=int,default=4)
parser.add_argument('-f','--column_for_lexical_items', help = 'column number in TSV for the tokens. Default=2', type=int, default=2)
parser.add_argument('-l','--language', help = 'language (used for special normalization step for fr, based on component categories found in the conll file). Default=fr', type=str, default = 'fr')
args = parser.parse_args()



def extract_annotated_mwes(input_stream, mweid_col, component_col, conll_lines, language):

    # key = 1 mwe (= 1 tuple of components)
    # val = nb occs in file
    mwes2nbocc = defaultdict(int)
    mwes2types = defaultdict(set)

    lines = input_stream.readlines()
    mweid2components = defaultdict(list) #dict key = mwe id, val = list of components
    mweid2type = defaultdict(str) #dict key = mwe id, val = type of mwe

    for i,line in enumerate(lines):
        line = line[:-1].strip()
        # if end of sentence
        if not line:
            for mweid in mweid2components:
                mwe = normalize_mwe( mweid2components[mweid] , language )  #tuple(mweid2components[mweid])
                mwes2nbocc[ mwe ] += 1
                mwes2types[ mwe ].add(mweid2type[mweid])
            # reset for next sentence
            mweid2components = defaultdict(list)
            mweid2type = defaultdict(str)
        else:
            cols = line.split("\t")

            if cols[mweid_col] != '_' and cols[mweid_col]:

                # component of the VMWE is lemma if conll file was provided
                if conll_lines:
                    conll_line = conll_lines[i].split('\t')
                    component = conll_line[2]
                    cat = conll_line[3]
                else:
                    component = cols[component_col]
                    cat = 'u'
                # French hack for the le/lui lemma
                if component == 'le/lui':
                    component = 'se'

                # split id in case of mwe embeddings
                mweids = cols[mweid_col].split(';')
                for full_mweid in mweids:
                    splitted_mweid = full_mweid.split(':', 1)
                    mweid = splitted_mweid[0]
                    if len(splitted_mweid) > 1:
                        mwe_type = splitted_mweid[1]
                        mweid2type[mweid] = mwe_type
                    mweid2components[mweid].append( (component,cat) ) 

    return (mwes2nbocc,mwes2types)

def normalize_mwe( componentcat_list , language ):
    """ input = list of component/cat pairs forming a VMWE, in the linear order found in sentence
    output = tuple of reordered components
    (reordering based on categories of the components, done for French only
    """
    if language == 'fr':
        cats = [ e[1] for e in componentcat_list ]
        if cats == ['N','V'] or cats == ['NOUN','VERB']:
            componentcat_list.reverse()

    return tuple( [ e[0] for e in componentcat_list ] )

def load_conll_file(conll_stream):
    # simply load the lines, they will be split iff they correspond to a component of a VMWE
    if conll_stream:
        lines = conll_stream.readlines()
        return lines
    return None

if __name__ == "__main__":

    component_col = args.column_for_lexical_items -1
    mweid_col = args.column_for_annotated_mwe_ids -1


    conll_lines = load_conll_file(args.conll_file)

    (mwes2nbocc, mwes2types) = extract_annotated_mwes(args.annotated_tabulated_file, mweid_col, component_col, conll_lines, args.language)

    total_nb_annotated_mwes = sum(mwes2nbocc.values())
    print("#TOTAL_NB_ANNOTATED_MWES="+str(total_nb_annotated_mwes))

    # print annotated mwes by reverse order of frequency, and alphabetically
    for mwe in sorted(mwes2nbocc, key=lambda x: (- mwes2nbocc[x], x)):
        print(" ".join(mwe) + "\t" + str(mwes2nbocc[mwe]) +"\t" + " ".join(mwes2types[mwe]))
